# SPDX-FileCopyrightText: Emery Hemingway
#
# SPDX-License-Identifier: LicenseRef-Hippocratic-1.1

let
  pkgOverlay = self: super:
    # Overlay of locally defined packages
    with self; {
      depot = callPackage ./depot { };
      dhallPackages = super.dhallPackages // (callPackage ./dhall { });
      genode = callPackage ./upstream { };
      nova = callPackage ./NOVA { };
    };

  toolchainOverlay = import ./toolchain-overlay;
  # Overlay of toolchain patches

  nixpkgs' = builtins.fetchGit {
    # A branch of Nixpkgs with a custom "crossSystem" mechanism
    url = "https://github.com/ehmry/nixpkgs.git";
    ref = "hybrid";
    rev = "3985d04b2cfe144beaf0fde58b2904763004197f";
  };

in { nixpkgs ? nixpkgs', dhall-haskell ? ./dhall-haskell.nix }:

import nixpkgs {
  # Evaluate an overlayed Nixpkgs for a Genode target
  config.allowUnsupportedSystem = true;
  crossSystem = {
    isx86_64 = true;
    isGenode = true;
    imports = [ ./platform.nix ];
  };
  overlays = [ (toolchainOverlay dhall-haskell) pkgOverlay ];
}
